@@ -2,51 +2,28 @@ package Catmandu::Store::ElasticSearch;
 
 use Catmandu::Sane;
 
-our $VERSION = '0.0512';
+our $VERSION = '1.0';
 
-use Moo;
 use Search::Elasticsearch;
 use Catmandu::Util qw(is_instance);
 use Catmandu::Store::ElasticSearch::Bag;
+use Moo;
 use namespace::clean;
 
 with 'Catmandu::Store';
-with 'Catmandu::Droppable';
 
-has index_name     => (is => 'ro', required => 1);
-has index_settings => (is => 'ro', lazy     => 1, default => sub {+{}});
-has index_mappings => (is => 'ro', lazy     => 1, default => sub {+{}});
-has _es_args       => (is => 'rw', lazy     => 1, default => sub {+{}});
-has es => (is => 'lazy');
-
-# used internally
-has is_es_1_or_2 => (is => 'lazy');
-
-sub _build_es {
-    my ($self) = @_;
-    my $es = Search::Elasticsearch->new($self->_es_args);
-    unless ($es->indices->exists(index => $self->index_name)) {
-        $es->indices->create(
-            index => $self->index_name,
-            body  => {
-                settings => $self->index_settings,
-                mappings => $self->index_mappings,
-            },
-        );
-    }
-    $es;
-}
+has _es_args     => (is => 'rw', lazy => 1, default => sub {+{}});
+has es           => (is => 'lazy');
+has is_es_1_or_2 => (is => 'lazy', init_arg => undef);
 
 sub BUILD {
     my ($self, $args) = @_;
-
-    # TODO filter out own args
     $self->_es_args($args);
 }
 
-sub drop {
+sub _build_es {
     my ($self) = @_;
-    $self->es->indices->delete(index => $self->index_name);
+    Search::Elasticsearch->new($self->_es_args);
 }
 
 sub _build_is_es_1_or_2 {
@@ -71,43 +48,43 @@ Catmandu::Store::ElasticSearch - A searchable store backed by Elasticsearch
     # From the command line
 
     # Import data into ElasticSearch
-    $ catmandu import JSON to ElasticSearch --index-name 'catmandu' < data.json
+    $ catmandu import JSON to ElasticSearch --bag catmandu < data.json
 
     # Export data from ElasticSearch
-    $ catmandu export ElasticSearch --index-name 'catmandu' to JSON > data.json
+    $ catmandu export ElasticSearch --bag catmandu to JSON > data.json
 
     # Export only one record
-    $ catmandu export ElasticSearch --index-name 'catmandu' --id 1234
+    $ catmandu export ElasticSearch --bag catmandu --id 1234
 
     # Export using an ElasticSearch query
-    $ catmandu export ElasticSearch --index-name 'catmandu' --query "name:Recruitment OR name:college"
+    $ catmandu export ElasticSearch --bag catmandu --query "name:Recruitment OR name:college"
 
     # Export using a CQL query (needs a CQL mapping)
-    $ catmandu export ElasticSearch --index-name 'catmandu' --q "name any college"
+    $ catmandu export ElasticSearch --bag catmandu --cql-query "name any college"
 
     # You need to specify the client version if your Elasticsearch server version is
     # not the same as your default Search::Elasticsearch client version
-    $ catmandu import JSON to ElasticSearch --index-name 'catmandu' --client '5_0::Direct' < data.json
+    $ catmandu import JSON to ElasticSearch --bag catmandu --client '5_0::Direct' < data.json
 
     # From Perl
 
     use Catmandu;
 
-    my $store = Catmandu->store('ElasticSearch', index_name => 'catmandu');
+    my $store = Catmandu->store('ElasticSearch');
 
-    my $obj1 = $store->bag->add({ name => 'Patrick' });
+    my $obj1 = $store->bag('catmandu')->add({ name => 'Patrick' });
 
     printf "obj1 stored as %s\n" , $obj1->{_id};
 
     # Force an id in the store
-    my $obj2 = $store->bag->add({ _id => 'test123' , name => 'Nicolas' });
+    my $obj2 = $store->bag('catmandu')->add({ _id => 'test123' , name => 'Nicolas' });
 
     # Commit all changes
-    $store->bag->commit;
+    $store->bag('catmandu')->commit;
 
-    $store->bag->delete('test123');
+    $store->bag('catmandu')->delete('test123');
 
-    $store->bag->delete_all;
+    $store->bag('catmandu')->delete_all;
 
     # All bags are iterators
     $store->bag->each(sub { ... });
@@ -132,28 +109,25 @@ Catmandu::Store::ElasticSearch - A searchable store backed by Elasticsearch
 
 =head1 METHODS
 
-=head2 new(index_name => $name, [...])
+=head2 new(%params)
 
-=head2 new(index_name => $name , index_mapping => \%map, [...])
+=head2 new(%params, bags => { mybag => { index => 'myindex', mapping => \%map cql_mapping => \%map } })
 
-=head2 new(index_name => $name , ... , bags => { data => { cql_mapping => \%map } })
+Create a new Catmandu::Store::ElasticSearch store. ElasticSearch connection
+parameters will be passed on to the underlying client.
 
-Create a new Catmandu::Store::ElasticSearch store connected to index $name.
-Optional extra ElasticSearch connection parameters will be passed on to the
-backend database.
-
-Optionally provide an C<index_mapping> which contains a ElasticSearch schema
-for each field in the index (See below).
-
-Optionally provide for each bag a C<type> to indicate which mapping to use.
+Optionally provide for each bag a C<index> to indicate which index to use.
 This defaults to the bag's name.
 
+Optionally provide for each bag a C<type> to indicate the name of the mapping.
+This defaults to the bag's name.
+
+Optionally provide for each bag a C<mapping> which contains a ElasticSearch schema
+for each field in the index (See below).
+
 Optionally provide for each bag a C<cql_mapping> to map fields to CQL indexes.
 
-=head2 drop
-
-Deletes the Elasticsearch index backing this store. Calling functions after
-this may fail until this class is reinstantiated, creating a new index.
+Optionally provide for each bag an C<on_error> error handler (See below).
 
 =head1 INHERITED METHODS
 
@@ -163,8 +137,6 @@ This Catmandu::Store implements:
 
 =item L<Catmandu::Store>
 
-=item L<Catmandu::Droppable>
-
 =back
 
 Each Catmandu::Bag in this Catmandu::Store implements:
@@ -181,32 +153,20 @@ Each Catmandu::Bag in this Catmandu::Store implements:
 
 =back
 
-=head1 INDEX MAP
+=head1 INDEX MAPPING
 
-The index_mapping contains a Elasticsearch schema mappings for each
+The mapping contains a Elasticsearch schema mappings for each
 bag defined in the index. E.g.
 
     {
-        data => {
-            properties => {
-                _id => {
-                    type           => 'string',
-                    include_in_all => 'true',
-                    index          => 'not_analyzed'
-                } ,
-                title => {
-                    type           => 'string'
-                }
+        properties => {
+            title => {
+                type => 'text'
             }
         }
     }
 
-In the example above the default 'data' bag of the ElasticSearch contains
-an '_id' field of type 'string' which is stored automatically also in the
-'_all' search field. The '_id' is not analyzed. The bag also contains a 'title'
-field of type string.
-
-See L<https://www.elastic.co/guide/en/elasticsearch/reference/2.2/mapping.html>
+See L<https://www.elastic.co/guide/en/elasticsearch/reference/current/mapping.html>
 for more information on mappings.
 
 These mappings can be passed inside a Perl program, or be written into a
@@ -217,24 +177,20 @@ Catmandu 'catmandu.yml' configuration file. E.g.
        search:
           package: ElasticSearch
           options:
-            index_name: catmandu
-            index_mappings
-              data:
-                properties:
-                    _id:
-                        type: string
-                        include_in_all: true
-                        index: not_analyzed
+            bags:
+              mybag:
+                mapping:
+                  properties:
                     title:
-                        type: string
+                      type: text
 
 Via the command line these configuration parameters can be read in by using the
 name of the store, C<search> in this case:
 
-   $ catmandu import JSON to search < data.json
-   $ catmandu export search to JSON > data.json
+   $ catmandu import JSON to search --bag mybag < data.json
+   $ catmandu export search --bag mybag to JSON > data.json
 
-=head1 CQL MAP
+=head1 CQL MAPPING
 
 Catmandu::Store::ElasticSearch supports CQL searches when a cql_mapping is provided
 for each bag. This hash contains a translation of CQL fields into Elasticsearch
@@ -292,34 +248,30 @@ Also this configuration can be added to a catmandu.yml configuration file like:
            package: ElasticSearch
            options:
              client: 6_0::Direct
-             index_name: catmandu
-             index_mappings:
-               book:
-                 properties:
-                     title:
-                         type: text
              bags:
-               data:
-                  type: book
-                  cql_mapping:
-                    indexes:
-                        title:
-                            op:
-                                'any': true
-                                'all': true
-                                '=':   true
-                                '<>':  true
-                                'exact':
-                                    field: [ 'mytitle.exact' , 'myalttitle.exact' ]
-                            field: mytitle
-                            sort: true
-                            cb: [ 'Biblio::Search' , 'normalize_title' ]
-                    }
+               book:
+                 mapping:
+                   properties:
+                     title:
+                       type: text
+                 cql_mapping:
+                   indexes:
+                       title:
+                           op:
+                               'any': true
+                               'all': true
+                               '=':   true
+                               '<>':  true
+                               'exact':
+                                   field: [ 'mytitle.exact' , 'myalttitle.exact' ]
+                           field: mytitle
+                           sort: true
+                           cb: [ 'Biblio::Search' , 'normalize_title' ]
 
 Via the command line these configuration parameters can be read in by using the
 name of the store, C<search> in this case:
 
-   $ catmandu export search -q 'title any blablabla' to JSON > data.json
+   $ catmandu export search --bag book -q 'title any blablabla' to JSON > data.json
 
 =head1 COMPATIBILITY
 
@@ -332,9 +284,9 @@ The appropriate client should be installed:
 
 And specified in the options:
 
-    Catmandu::Store::ElasticSearch->new(index_name => 'myindex', client => '1_0::Direct')
+    Catmandu::Store::ElasticSearch->new(client => '1_0::Direct')
 
-If you want to use the C<delete_by_query> method with Elasticsearch >= 2.0 you
+If you want to use the C<delete_by_query> method with Elasticsearch 2.0 you
 have to L<install the delete by query plugin|https://www.elastic.co/guide/en/elasticsearch/plugins/current/plugins-delete-by-query.html>.
 
 =head1 ERROR HANDLING
@@ -348,8 +300,7 @@ a store. E.g. to create an error handler for the bag 'data' index use:
     };
 
     my $store = Catmandu::Store::ElasticSearch->new(
-        index_name => 'catmandu'
-        bags       => { data => { on_error => $error_handler } }
+        bags => { data => { on_error => $error_handler } }
     });
 
 Instead of a callback, the following shortcuts are also accepted for on_error:
@@ -361,17 +312,29 @@ throw: throw the response as an error
 ignore: do nothing
 
     my $store = Catmandu::Store::ElasticSearch->new(
-        index_name => 'catmandu'
-        bags       => { data => { on_error => 'log' } }
+        bags => { data => { on_error => 'log' } }
     });
 
+=head1 UPGRADING FROM A PRE 1.0 VERSION
+
+Versions of this store < 1.0 used Elasticsearch types to map bags to a single
+index. Support for multiple types in one index has since been removed from
+Elasticsearch and since 1.0 each bag is mapped to an index.
+
+You need to export you data before upgrading, update the configuration and then
+import you data again.
+
 =head1 SEE ALSO
 
 L<Catmandu::Store>
 
 =head1 AUTHOR
 
-Nicolas Steenlant, C<< <nicolas.steenlant at ugent.be> >>
+=over 4
+
+=item Nicolas Steenlant, C<< <nicolas.steenlant at ugent.be> >>
+
+=back
 
 =head1 CONTRIBUTORS
 
