@@ -2,24 +2,47 @@ package Catmandu::Store::ElasticSearch::Bag;
 
 use Catmandu::Sane;
 
-our $VERSION = '0.0512';
+our $VERSION = '1.0';
 
-use Moo;
 use Catmandu::Hits;
 use Cpanel::JSON::XS qw(encode_json decode_json);
 use Catmandu::Store::ElasticSearch::Searcher;
 use Catmandu::Store::ElasticSearch::CQL;
 use Catmandu::Util qw(is_code_ref is_string);
+use Moo;
+use namespace::clean;
 
 with 'Catmandu::Bag';
 with 'Catmandu::Droppable';
 with 'Catmandu::CQLSearchable';
 
-has type        => (is => 'ro', lazy => 1);
-has buffer_size => (is => 'ro', lazy => 1, builder => 'default_buffer_size');
-has _bulk       => (is => 'ro', lazy => 1);
+has index       => (is => 'lazy');
+has settings    => (is => 'lazy');
+has mapping     => (is => 'lazy');
+has type        => (is => 'lazy');
+has buffer_size => (is => 'lazy', builder => 'default_buffer_size');
+has _bulk       => (is => 'lazy');
 has cql_mapping => (is => 'ro');
-has on_error => (is => 'ro', default => sub {'log'});
+has on_error    => (is => 'lazy');
+
+sub BUILD {
+    $_[0]->create_index;
+}
+
+sub create_index {
+    my ($self) = @_;
+    my $es = $self->store->es;
+    unless ($es->indices->exists(index => $self->index)) {
+        $es->indices->create(
+            index => $self->index,
+            body  => {
+                settings => $self->settings,
+                mappings => {$self->type => $self->mapping},
+            },
+        );
+    }
+    1;
+}
 
 sub default_buffer_size {100}
 
@@ -49,6 +72,22 @@ sub _coerce_on_error {
         "on_error should be code ref, 'throw', 'log', or 'ignore'");
 }
 
+sub _build_on_error {
+    'log';
+}
+
+sub _build_settings {
+    +{};
+}
+
+sub _build_mapping {
+    +{};
+}
+
+sub _build_index {
+    $_[0]->name;
+}
+
 sub _build_type {
     $_[0]->name;
 }
@@ -57,7 +96,7 @@ sub _build__bulk {
     my ($self)   = @_;
     my $on_error = $self->_coerce_on_error($self->on_error);
     my %args     = (
-        index     => $self->store->index_name,
+        index     => $self->index,
         type      => $self->type,
         max_count => $self->buffer_size,
         on_error  => $on_error,
@@ -77,9 +116,9 @@ sub generator {
     sub {
         state $scroll = do {
             my %args = (
-                index => $self->store->index_name,
+                index => $self->index,
                 type  => $self->type,
-                size => $self->buffer_size,  # TODO divide by number of shards
+                size  => $self->buffer_size, # TODO divide by number of shards
                 body => {query => {match_all => {}},},
             );
             if ($self->store->is_es_1_or_2) {
@@ -99,17 +138,15 @@ sub generator {
 
 sub count {
     my ($self) = @_;
-    $self->store->es->count(
-        index => $self->store->index_name,
-        type  => $self->type,
-    )->{count};
+    $self->store->es->count(index => $self->index, type => $self->type,)
+        ->{count};
 }
 
 sub get {
     my ($self, $id) = @_;
     try {
         my $data = $self->store->es->get_source(
-            index => $self->store->index_name,
+            index => $self->index,
             type  => $self->type,
             id    => $id,
         );
@@ -136,7 +173,7 @@ sub delete_all {
     my $es = $self->store->es;
     if ($es->can('delete_by_query')) {
         $es->delete_by_query(
-            index => $self->store->index_name,
+            index => $self->index,
             type  => $self->type,
             body  => {query => {match_all => {}},},
         );
@@ -144,11 +181,8 @@ sub delete_all {
     else {    # TODO document plugin needed for es 2.x
         $es->transport->perform_request(
             method => 'DELETE',
-            path   => '/'
-                . $self->store->index_name . '/'
-                . $self->type
-                . '/_query',
-            body => {query => {match_all => {}},}
+            path   => '/' . $self->index . '/' . $self->type . '/_query',
+            body   => {query => {match_all => {}},}
         );
     }
 }
@@ -158,7 +192,7 @@ sub delete_by_query {
     my $es = $self->store->es;
     if ($es->can('delete_by_query')) {
         $es->delete_by_query(
-            index => $self->store->index_name,
+            index => $self->index,
             type  => $self->type,
             body  => {query => $args{query},},
         );
@@ -166,11 +200,8 @@ sub delete_by_query {
     else {    # TODO document plugin needed for es 2.x
         $es->transport->perform_request(
             method => 'DELETE',
-            path   => '/'
-                . $self->store->index_name . '/'
-                . $self->type
-                . '/_query',
-            body => {query => $args{query},}
+            path   => '/' . $self->index . '/' . $self->type . '/_query',
+            body   => {query => $args{query},}
         );
     }
 }
@@ -180,7 +211,7 @@ sub commit {
     $self->_bulk->flush;
     $self->store->es->transport->perform_request(
         method => 'POST',
-        path   => '/' . $self->store->index_name . '/_refresh',
+        path   => '/' . $self->index . '/_refresh',
     );
 }
 
@@ -198,7 +229,7 @@ sub search {
     }
 
     my $res = $self->store->es->search(
-        index => $self->store->index_name,
+        index => $self->index,
         type  => $self->type,
         body  => {%args, from => $start, size => $limit,},
     );
@@ -212,15 +243,18 @@ sub search {
         $hits->{hits} = [map {$bag->get($_->{_id})} @$docs];
     }
     elsif ($args{fields}) {
+
         # TODO check if fields includes id_key
         $hits->{hits} = [map {$_->{fields} || +{}} @$docs];
     }
     else {
-        $hits->{hits} = [map {
-            my $data = $_->{_source};
-            $data->{$id_key} = $_->{_id};
-            $data;
-        } @$docs];
+        $hits->{hits} = [
+            map {
+                my $data = $_->{_source};
+                $data->{$id_key} = $_->{_id};
+                $data;
+            } @$docs
+        ];
     }
 
     $hits = Catmandu::Hits->new($hits);
@@ -309,8 +343,7 @@ sub normalize_sort {
 
 sub drop {
     my ($self) = @_;
-    $self->delete_all;
-    $self->commit;
+    $self->store->es->indices->delete(index => $self->index);
 }
 
 1;
@@ -325,10 +358,22 @@ Catmandu::Store::ElasticSearch::Bag - Catmandu::Bag implementation for Elasticse
 
 =head1 DESCRIPTION
 
-This class isn't normally used directly. Instances are constructed using the store's C<bag> method.
+See the main documentation at L<Catmandu::Store::ElasticSearch>.
+
+=head1 METHODS
+
+This class inherits all the methods of L<Catmandu::Bag>,
+L<Catmandu::CQLSearchable> and L<Catmandu::Droppable>.
+It also provides the following methods:
+
+=head2 create_index()
+
+This method is called automatically when the bag is instantiated. You only need
+to call it manually it after deleting the index with C<drop> or the
+Elasticsearch API.
 
 =head1 SEE ALSO
 
-L<Catmandu::Bag>, L<Catmandu::Searchable>
+L<Catmandu::Bag>, L<Catmandu::Searchable>, L<Catmandu::CQLSearchable>, L<Catmandu::Droppable>
 
 =cut
